:root {
  --bg: #fff;
  --clr: #4d5156;
  --dark: #f3f3f3;
  --a: #0077cc;
}
@media (prefers-color-scheme: dark) {
  :root {
    --bg: #35363a;
    --clr: #b1b1b1;
    --dark: #202124;
    --a: #0797ff;
  }
}

body {
  background-color: var(--bg);
  color: var(--clr);
  font-family: "Helvetica Neue", Helvetica, sans-serif;
  font-size: 13px;
  max-width: 80%;
  margin: auto;
  margin: 10px auto;
}

@media screen and (max-width: 600px) {
  body {
    margin: 10px;
    max-width: unset;
  }
}

button,
textarea,
input {
  border: solid 1px rgba(0, 0, 0, 0.25);
  outline: none;
}
button:focus,
textarea:focus,
input:focus {
  background-color: #e7f3ff;
}

button,
input[type=submit],
input[type=button] {
  height: 28px;
  color: #444;
  background-image: linear-gradient(rgb(237, 237, 237), rgb(237, 237, 237) 38%, rgb(222, 222, 222));
  box-shadow: rgba(0, 0, 0, 0.08) 0 1px 0, rgba(255, 255, 255, 0.75) 0 1px 2px inset;
  text-shadow: rgb(240, 240, 240) 0 1px 0;
  cursor: pointer;
}
input[type=button]:disabled {
  opacity: 0.5;
}
input[type=text],
input[type=number] {
  height: 24px;
}
input[type=number] {
  width: 60px;
  box-sizing: border-box;
}
th {
  height: 32px;
  background-color: var(--dark);
  text-indent: 10px;
}
td:nth-child(3) {
  text-align: right;
}
textarea {
  width: 100%;
  box-sizing: border-box;
  padding: 5px;
}
ol {
  padding: 0 0 0 20px;
  font-size: 12px;
}
a {
  color: var(--a);
}

.wnt {
  color: #61611f;
  background-color: #ffffed;
  border: solid 1px #babaab;
  padding: 10px;
  margin: 15px 0;
  text-align: center;
}

.disabled {
  opacity: 0.2;
  pointer-events: none;
}

#rate {
  display: grid;
  grid-template-columns: 1fr min-content;
  margin: 10px 0;
}
